var TEXTAREA;
var PLANE;
var TARGET;
var URN;
var URN_TOP;


const INITIAL_PLANE_TOP = 12;
const INITIAL_PLANE_LEFT = 130;
const INITIAL_PLANE_ANGLE = degToRad(43);

const PLANE_ANIM_DURATION = 1500;

const R = 10;


function degToRad(deg) {
    return deg * Math.PI / 180;
}


function radToDeg(rad) {
    return rad * 180 / Math.PI;
}


function validateSubmission() {
    return TEXTAREA.value != "" && TEXTAREA.value.length <= 1500;
}


function animatePlaneIntoUrn(callback) {
    const rectPlaneStart = PLANE.getBoundingClientRect();
    const rectTarget = TARGET.getBoundingClientRect();
    const x1 = rectTarget.left - rectPlaneStart.left;
    const y1 = -(rectTarget.top - rectPlaneStart.top);
    const x2 = -R * Math.cos(INITIAL_PLANE_ANGLE - .5 * Math.PI);
    const y2 = -R * Math.sin(INITIAL_PLANE_ANGLE - .5 * Math.PI);
    const r3 = .5 * (x1 - x2 - R);
    const x3 = x2 + R + r3;
    const lengthA = (.5 * Math.PI - INITIAL_PLANE_ANGLE) * R;
    const lengthB = Math.PI * r3;
    const lengthC = y2 - y1;
    const lengthTotal = lengthA + lengthB + lengthC;

    let start;

    function step(timestamp) {
        // Compute progress
        if (start == null) start = timestamp;
        const t = Math.pow(Math.max(0, Math.min(1, (timestamp - start) / PLANE_ANIM_DURATION)), 3);

        // Compute (x, y) coordinates and plane angle
        let x, y, phi;
        if (t <= lengthA / lengthTotal) {
            const s = t * lengthTotal / lengthA;
            const alpha = (1 - s) * (1.5 * Math.PI + INITIAL_PLANE_ANGLE) + s * Math.PI * 2;
            x = x2 + R * Math.cos(alpha);
            y = y2 + R * Math.sin(alpha);
            phi = (1 - s) * INITIAL_PLANE_ANGLE + s * .5 * Math.PI;
        } else if (t <= (lengthA + lengthB) / lengthTotal) {
            const s = (t - lengthA / lengthTotal) * lengthTotal / lengthB;
            const alpha = (1 - s) * Math.PI + s * 0;
            x = x3 + r3 * Math.cos(alpha);
            y = y2 + r3 * Math.sin(alpha);
            phi = (1 - s) * .5 * Math.PI + s * .5 * (-Math.PI);
        } else {
            const s = (t - (lengthA + lengthB) / lengthTotal) * lengthTotal / lengthC;
            x = x1;
            y = (1 - s) * y2 + s * y1;
            phi = -.5 * Math.PI;
        }

        // Update the plane position
        PLANE.style.top = (INITIAL_PLANE_TOP - y) + "px";
        PLANE.style.left = (INITIAL_PLANE_LEFT + x) + "px";
        PLANE.style.transform = "rotate(" + radToDeg(INITIAL_PLANE_ANGLE - phi) + "deg)";

        // What to do next
        if (t < 1) {
            window.requestAnimationFrame(step);
        } else if (callback) {
            callback();
        }
    }

    window.requestAnimationFrame(step);
}


function handleShareButtonClick() {
    if (!validateSubmission()) {
        PLANE.classList.add("wiggle-strong");
        TEXTAREA.classList.add("has-error");
        setTimeout(() => {
            PLANE.classList.remove("wiggle-strong");
            TEXTAREA.classList.remove("has-error");
        }, 500);
        return;
    }
    URN_TOP.style.opacity = 0;
    animatePlaneIntoUrn(() => {
        URN_TOP.style.opacity = 1;
        PLANE.style.opacity = 0;
        URN.classList.add("wiggle");
        setTimeout(() => {
            URN.classList.remove("wiggle");
        }, 500);
        postIdea((data) => {
            let btn = document.getElementById("send");
            let btnText = document.getElementById("send-text");
            btnText.style.paddingRight = "0";
            if (data.success) {
                btnText.innerHTML = "Merci&nbsp;!";
                btn.classList.add("bg-success");
                btn.classList.add("text-secondary");
            } else {
                btnText.innerHTML = "Boîte pleine &#128532;";
                btn.classList.add("bg-error");
                btn.classList.add("text-secondary");
            }
            setTimeout(() => {
                btnText.style.paddingRight = "2em";
                btnText.innerHTML = "Proposer";
                btn.classList.remove("bg-success");
                btn.classList.remove("bg-error");
                btn.classList.remove("text-secondary");
                TEXTAREA.value = "";
                PLANE.style.top = INITIAL_PLANE_TOP + "px";
                PLANE.style.left = INITIAL_PLANE_LEFT + "px";
                PLANE.style.opacity = 1;
                PLANE.style.transform = "rotate(0deg)";
                document.getElementById("charcount").textContent = "0";
            }, 1000);
        });
    });
}


function handleUrnClick() {
    // TODO: an animation of a paper sheet coming out of the urn.
    URN.classList.add("wiggle-wait");
    URN_TOP.style.opacity = 0;
    setTimeout(() => {
        fetch(GET_IDEA_URL)
            .then(response => response.text())
            .then(data => {
                document.querySelector("#modal-idea .modal-body").textContent = data;
                document.getElementById("modal-idea").classList.add("active");
                URN.classList.remove("wiggle-wait");
                URN_TOP.style.opacity = 1;
            });
    }, 300);
}


function postIdea(callback) {
    const request = new Request(POST_IDEA_URL, {
        headers: {
            "X-CSRFToken": CSRF_TOKEN
        }
    });
    let formData = new FormData();
    formData.append("text", TEXTAREA.value);
    fetch(request, {
            method: "POST",
            body: formData,
            mode: "same-origin"
        })
        .then(response => response.json())
        .then(data => {
            if (callback) {
                callback(data);
            }
        });
}


window.addEventListener("load", () => {
    TEXTAREA = document.getElementById("input");
    PLANE = document.getElementById("plane");
    TARGET = document.getElementById("plane-target");
    URN = document.getElementById("urn");
    URN_TOP = document.getElementById("urn-top");
    document.getElementById("send").addEventListener("click", handleShareButtonClick);
    TEXTAREA.addEventListener("input", () => {
        document.getElementById("charcount").textContent = TEXTAREA.value.length;
    });
    URN.addEventListener("click", handleUrnClick);
});