import django.shortcuts
import django.http
import django.core.paginator
import django.utils
from piweb.decorators import require_app_access
from . import models


IDEA_COUNT_LIMIT = 200


def landing(request):
    return django.shortcuts.render(request, "nanowrimo/landing.html", {})


@require_app_access("nanowrimo")
def backoffice(request):
    query = models.Idea.objects.order_by("-date_submitted")
    items_per_page = 25
    paginator = django.core.paginator.Paginator(query, items_per_page)
    page_number = request.GET.get('page')
    ideas = paginator.get_page(page_number)
    return django.shortcuts.render(request, "nanowrimo/backoffice.html", {
        "ideas": ideas
    })


def post_idea(request):
    success = False
    if request.method == "POST":
        text = request.POST.get("text")
        if text is not None:
            if models.Idea.objects.all().count() <= IDEA_COUNT_LIMIT:
                models.Idea.objects.create(content=text)
                success = True
    return django.http.JsonResponse({
        "success": success
    }, safe=False)


def get_idea(request):
    query = models.Idea.objects.filter(accepted=True)
    if query.exists():
        return django.http.HttpResponse(
            query.order_by("?").first().content,
            content_type="text/plain"
        )
    else:
        return django.http.HttpResponse(
            "La boîte est vide 😥",
            content_type="text/plain; charset=utf-8",
        )
        

@require_app_access("nanowrimo")
def toggle_idea(request):
    success = False
    new_status = None
    if request.method == "POST":
        iid = request.POST.get("iid")
        if iid is not None:
            query = models.Idea.objects.filter(id=int(iid))
            if query.exists():
                item = query.get()
                item.accepted = not item.accepted
                new_status = item.accepted
                item.date_accepted = django.utils.timezone.now()
                item.save()
                success = True
    return django.http.JsonResponse({
        "success": success,
        "new_status": new_status
    }, safe=False)


@require_app_access("nanowrimo")
def delete_idea(request):
    success = False
    if request.method == "POST":
        iid = request.POST.get("iid")
        if iid is not None:
            query = models.Idea.objects.filter(id=int(iid))
            if query.exists():
                item = query.get()
                item.delete()
                success = True
    return django.http.JsonResponse({
        "success": success,
    }, safe=False)